<?php

namespace Drupal\skilling\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\skilling\SkillingConstants;
use Drupal\skilling\SkillingCurrentUser;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\skilling\Utilities as SkillingUtilities;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Render\Renderer;
use Drupal\skilling\History;

/**
 * Class SubmissionController.
 */
class SubmissionController extends ControllerBase {

  const EXERCISE_SUBMISSION_LINK_TEXT = 'Submit solution';
  const NOT_STUDENT_MESSAGE = 'If you were a student... config.';

  /**
   * The current user service.
   *
   * @var \Drupal\skilling\SkillingCurrentUser
   */
  protected $currentUserService;

  /**
   * @var EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var SkillingUtilities
   */
  protected $skillingUtilities;

  /**
   * @var ConfigFactory
   */
  protected $configFactory;

  /**
   * @var Renderer
   */
  protected $renderer;

  /**
   * The Skilling history event recorder.
   *
   * @var \Drupal\skilling\History
   */
  protected $historyService;

  /**
   * Constructs a new SubmissionController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\skilling\Utilities $skillingUtilities
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   * @param \Drupal\Core\Render\Renderer $renderer
   * @param \Drupal\skilling\SkillingCurrentUser $currentUserService
   *   The current user service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, SkillingUtilities $skillingUtilities, ConfigFactory $configFactory, Renderer $renderer, SkillingCurrentUser $currentUserService,  History $historyService) {
    $this->entityTypeManager = $entityTypeManager;
    $this->skillingUtilities = $skillingUtilities;
    $this->configFactory = $configFactory;
    $this->renderer = $renderer;
    $this->currentUserService = $currentUserService;
    $this->historyService = $historyService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /* @noinspection PhpParamsInspection */
    return new static(
      $container->get('entity_type.manager'),
      $container->get('skilling.utilities'),
      $container->get('config.factory'),
      $container->get('renderer'),
      $container->get('skilling.skilling_current_user'),
      $container->get('skilling.history')
    );
  }

  /**
   * Close popup submission window. This is the destination
   * of popup (floating) submissions windows.
   *
   * @param $operation
   * @param $exercise_nid
   * @param $submission_nid
   *
   * @return \Symfony\Component\HttpFoundation\Response
   * @throws \Exception
   */
  public function closeSubmissionWindow($operation, $exercise_nid, $submission_nid) {
    $renderable = [
      '#theme' => 'update_opener_and_close', //Template to use.
      '#operation' => $operation,
      '#exercise_nid' => $exercise_nid,
      '#submission_nid' => $submission_nid,
    ];
    $rendered = $this->renderer->render($renderable);
    return new Response($rendered);
  }

  /**
   * Make a message  that a submission link is out of date.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   * @throws \Exception
   */
  public function getSubmissionChangedMessage() {
    $settings = $this->configFactory->get('skilling.settings');
    $message = $settings->get('submissions.changed_message');
    $renderable = [
      '#theme' => 'submission_link_changed',
    ];
    $rendered = $this->renderer->render($renderable);
    $result = [
      'message' => $rendered,
    ];
    return new JsonResponse($result);
  }

  public function showCompletionScore() {
    return [
      '#markup' => $this->t('showCompletionScore'),
    ];
  }

  /**
   * Show a student his/her own submissions.
   *
   * The code loads and shows a view, rather than let Views handle the
   * route and menu item itself. The view uses a contextual filter for
   * the user the submissions are from. The filter uses a default argument
   * of the current user's uid. Handling the route here prevents
   * users from appending a uid to the URL, and getting another user's
   * submission.
   *
   * @return array|\Drupal\Core\StringTranslation\TranslatableMarkup
   *   Renderable.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\skilling\Exception\SkillingNotFoundException
   * @throws \Drupal\skilling\Exception\SkillingUnknownValueException
   */
  public function showSubmissionsForStudent() {
    if (!$this->currentUserService->isStudent()) {
      return [
        '#markup' => $this->t('Sorry, this view is for students only.'),
      ];
    }
    /** @var \Drupal\views\ViewExecutable $view */
    $view = Views::getView('submissions');
    $result = $this->t('Submissions attachment error.');
    if (is_object($view)) {
      $view->setDisplay('student_submissions_report_embed');
      $view->preExecute();
      $view->execute();
      $result = $view->buildRenderable('student_submissions_report_embed');
      // Record the event.
      $this->historyService->createHistoryNode(
        t('View submissions list'), SkillingConstants::HISTORY_EVENT_VIEW_SUBMISSIONS_LIST,
        $this->currentUserService->id(), [], ''
      );
    }
    return $result;
  }

}
